@@ -26,6 +26,7 @@
 #include <gsf-config.h>
 #include <gsf/gsf-infile-tar.h>
 #include <gsf/gsf.h>
+#include <glib/gi18n-lib.h>
 
 #include <string.h>
 
@@ -44,6 +45,8 @@ static GObjectClass *parent_class;
 typedef struct {
 	char *name;
 
+	GDateTime *modtime;
+
 	/* The location of data.  */
 	gsf_off_t offset;
 	gsf_off_t length;
@@ -92,10 +95,10 @@ typedef struct {
 #define GSF_INFILE_TAR_CLASS(k)    (G_TYPE_CHECK_CLASS_CAST ((k), GSF_INFILE_TAR_TYPE, GsfInfileTarClass))
 #define GSF_IS_INFILE_TAR_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GSF_INFILE_TAR_TYPE))
 
-static gsf_off_t
+static gint64
 unpack_octal (GsfInfileTar *tar, const char *s, size_t len)
 {
-	gsf_off_t res = 0;
+	gint64 res = 0;
 
 	while (len--) {
 		unsigned char c = *s++;
@@ -103,7 +106,7 @@ unpack_octal (GsfInfileTar *tar, const char *s, size_t len)
 			break;
 		if (c < '0' || c > '7') {
 			tar->err = g_error_new (gsf_input_error_id (), 0,
-						"Invalid tar header");
+						_("Invalid tar header"));
 			return 0;
 		}
 		res = (res << 3) + (c - '0');
@@ -121,6 +124,7 @@ tar_create_dir (GsfInfileTar *dir, const char *name)
 	c.offset = 0;
 	c.length = 0;
 	c.name = g_strdup (name);
+	c.modtime = NULL;
 	c.dir = g_object_new (GSF_INFILE_TAR_TYPE, NULL);
 
 	/*
@@ -206,10 +210,11 @@ tar_init_info (GsfInfileTar *tar)
 		char *name;
 		gsf_off_t length;
 		gsf_off_t offset;
+		gint64 mtime;
 
 		if (memcmp (header->filler, end.filler, sizeof (end.filler))) {
 			tar->err = g_error_new (gsf_input_error_id (), 0,
-						"Invalid tar header");
+						_("Invalid tar header"));
 			break;
 		}
 
@@ -224,9 +229,7 @@ tar_init_info (GsfInfileTar *tar)
 		length = unpack_octal (tar, header->size, sizeof (header->size));
 		offset = gsf_input_tell (tar->source);
 
-#if 0
-		g_printerr ("[%s]: %d\n", name, (int)length);
-#endif
+		mtime = unpack_octal (tar, header->mtime, sizeof (header->mtime));
 
 		switch (header->typeflag) {
 		case '0': case 0: {
@@ -239,6 +242,9 @@ tar_init_info (GsfInfileTar *tar)
 			while ((s = strchr (n, '/')))
 				n = s + 1;
 			c.name = g_strdup (n);
+			c.modtime = mtime > 0
+				? g_date_time_new_from_unix_utc (mtime)
+				: NULL;
 			c.offset = offset;
 			c.length = length;
 			c.dir = NULL;
@@ -257,14 +263,14 @@ tar_init_info (GsfInfileTar *tar)
 			if (pending_longname ||
 			    strcmp (name, MAGIC_LONGNAME) != 0) {
 				tar->err = g_error_new (gsf_input_error_id (), 0,
-							"Invalid longname header");
+							_("Invalid longname header"));
 				break;
 			}
 
 			n = gsf_input_read (tar->source, length, NULL);
 			if (!n) {
 				tar->err = g_error_new (gsf_input_error_id (), 0,
-							"Failed to read longname");
+							_("Failed to read longname"));
 				break;
 			}
 
@@ -284,7 +290,7 @@ tar_init_info (GsfInfileTar *tar)
 		if (!tar->err &&
 		    gsf_input_seek (tar->source, offset + length, G_SEEK_SET)) {
 			tar->err = g_error_new (gsf_input_error_id (), 0,
-						"Seek failed");
+						_("Seek failed"));
 			break;
 		}
 	}
@@ -292,7 +298,7 @@ tar_init_info (GsfInfileTar *tar)
 	if (pending_longname) {
 		if (!tar->err)
 			tar->err = g_error_new (gsf_input_error_id (), 0,
-						"Truncated archive");
+						_("Truncated archive"));
 		g_free (pending_longname);
 	}
 
@@ -323,6 +329,7 @@ gsf_infile_tar_dup (GsfInput *src_input, GError **err)
 		/* This copies the structure.  */
 		TarChild c = g_array_index (src->children, TarChild, ui);
 		c.name = g_strdup (c.name);
+		if (c.modtime) g_date_time_ref (c.modtime);
 		if (c.dir) g_object_ref (c.dir);
 		g_array_append_val (res->children, c);
 	}
@@ -371,6 +378,7 @@ gsf_infile_tar_child_by_index (GsfInfile *infile, int target, GError **err)
 		GsfInput *input = gsf_input_proxy_new_section (tar->source,
 							       c->offset,
 							       c->length);
+		gsf_input_set_modtime (input, c->modtime);
 		gsf_input_set_name (input, c->name);
 		return input;
 	}
@@ -432,6 +440,8 @@ gsf_infile_tar_dispose (GObject *obj)
 	for (ui = 0; ui < tar->children->len; ui++) {
 		TarChild *c = &g_array_index (tar->children, TarChild, ui);
 		g_free (c->name);
+		if (c->modtime)
+			g_date_time_unref (c->modtime);
 		if (c->dir)
 			g_object_unref (c->dir);
 	}
@@ -538,8 +548,8 @@ gsf_infile_tar_class_init (GObjectClass *gobject_class)
 		(gobject_class,
 		 PROP_SOURCE,
 		 g_param_spec_object ("source",
-				      "Source",
-				      "The archive being interpreted.",
+				      _("Source"),
+				      _("The archive being interpreted"),
 				      GSF_INPUT_TYPE,
 				      GSF_PARAM_STATIC |
 				      G_PARAM_READWRITE |
@@ -553,7 +563,7 @@ GSF_CLASS (GsfInfileTar, gsf_infile_tar,
 /**
  * gsf_infile_tar_new:
  * @source: A base #GsfInput
- * @err: A #GError, optionally %null
+ * @err: A #GError, optionally %NULL
  *
  * Opens the root directory of a Tar file.
  * <note>This adds a reference to @source.</note>
