@@ -381,7 +381,7 @@ gsf_input_read (GsfInput *input, size_t num_bytes, guint8 *optional_buffer)
 }
 
 /**
- * gsf_input_read0:
+ * gsf_input_read0: (rename-to gsf_input_read):
  * @input: the input stream
  * @num_bytes: (in): number of bytes to read
  * @bytes_read: (out): copy of @num_bytes
@@ -391,8 +391,6 @@ gsf_input_read (GsfInput *input, size_t num_bytes, guint8 *optional_buffer)
  *
  * Returns: (array length=bytes_read) (element-type guint8) (transfer full):
  * the data read.
- *
- * Rename to: gsf_input_read
  **/
 
 guint8 *
@@ -405,7 +403,7 @@ gsf_input_read0 (GsfInput *input, size_t num_bytes, size_t *bytes_read)
 
 	*bytes_read = num_bytes;
 
-	if (num_bytes > (size_t)gsf_input_remaining (input))
+	if (num_bytes < 0 || (gsf_off_t)num_bytes > gsf_input_remaining (input))
 		return NULL;
 
 	res = g_new (guint8, num_bytes);
@@ -679,20 +677,19 @@ gsf_input_error (void)
 gboolean
 gsf_input_copy (GsfInput *input, GsfOutput *output)
 {
-	gsf_off_t    remaining = 0;
-	gsf_off_t    toread    = 0;
-	const guint8 * buffer  = NULL;
-	gboolean     success   = TRUE;
+	gboolean success = TRUE;
+	gsf_off_t remaining;
 
 	g_return_val_if_fail (input != NULL, FALSE);
 	g_return_val_if_fail (output != NULL, FALSE);
 
-	while ((remaining = gsf_input_remaining (input)) > 0 && (success)) {
-		toread = MIN (remaining, GSF_READ_BUFSIZE);
-		if (NULL == (buffer = gsf_input_read (input, toread, NULL)))
-			success = FALSE;
-		else
+	while (success && (remaining = gsf_input_remaining (input)) > 0) {
+		gsf_off_t toread = MIN (remaining, GSF_READ_BUFSIZE);
+		const guint8 * buffer = gsf_input_read (input, toread, NULL);
+		if (buffer)
 			success = gsf_output_write (output, toread, buffer);
+		else
+			success = FALSE;
 	}
 
 	return success;
