@@ -22,9 +22,19 @@
 #include <gsf-config.h>
 #include <gsf/gsf-output.h>
 #include <gsf/gsf.h>
+#include <glib/gi18n-lib.h>
 
 #include <string.h>
 
+/*
+ * FIXME!
+ *
+ * We cannot extend GsfInput, so for now we hang this on the object as
+ * an attribute.
+ */
+#define MODTIME_ATTR "GsfOutput::modtime"
+
+
 static gsf_off_t gsf_output_real_vprintf (GsfOutput *output,
 	char const* format, va_list args) G_GNUC_PRINTF (2, 0);
 
@@ -37,16 +47,29 @@ enum {
 	PROP_NAME,
 	PROP_SIZE,
 	PROP_CLOSED,
-	PROP_POS
+	PROP_POS,
+	PROP_MODTIME,
+	PROP_CONTAINER
 };
 
 static void
 gsf_output_set_property (GObject      *object,
 			 guint         property_id,
-	 G_GNUC_UNUSED   GValue const *value,
+			 GValue const *value,
 			 GParamSpec   *pspec)
 {
+	GsfOutput *output = GSF_OUTPUT (object);
+
 	switch (property_id) {
+	case PROP_NAME:
+		gsf_output_set_name (output, g_value_get_string (value));
+		break;
+	case PROP_MODTIME:
+		gsf_output_set_modtime (output, g_value_get_boxed (value));
+		break;
+	case PROP_CONTAINER:
+		gsf_output_set_container (output, g_value_get_object (value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -59,19 +82,28 @@ gsf_output_get_property (GObject     *object,
 			 GValue      *value,
 			 GParamSpec  *pspec)
 {
+	GsfOutput *output = GSF_OUTPUT (object);
+
 	/* gsf_off_t is typedef'd to gint64 */
+
 	switch (property_id) {
 	case PROP_NAME:
-		g_value_set_string (value, gsf_output_name (GSF_OUTPUT (object)));
+		g_value_set_string (value, gsf_output_name (output));
 		break;
 	case PROP_SIZE:
-		g_value_set_int64 (value, gsf_output_size (GSF_OUTPUT (object)));
-		break;
-	case PROP_POS:
-		g_value_set_int64 (value, gsf_output_tell (GSF_OUTPUT (object)));
+		g_value_set_int64 (value, gsf_output_size (output));
 		break;
 	case PROP_CLOSED:
-		g_value_set_boolean (value, gsf_output_is_closed (GSF_OUTPUT (object)));
+		g_value_set_boolean (value, gsf_output_is_closed (output));
+		break;
+	case PROP_POS:
+		g_value_set_int64 (value, gsf_output_tell (output));
+		break;
+	case PROP_MODTIME:
+		g_value_set_boxed (value, gsf_output_get_modtime (output));
+		break;
+	case PROP_CONTAINER:
+		g_value_set_object (value, output->container);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -91,6 +123,7 @@ gsf_output_dispose (GObject *obj)
 
 	gsf_output_set_container (output, NULL);
 	gsf_output_set_name (output, NULL);
+	gsf_output_set_modtime (output, NULL);
 
 	g_free (output->printf_buf);
 	output->printf_buf = NULL;
@@ -128,34 +161,94 @@ gsf_output_class_init (GObjectClass *gobject_class)
 
 	parent_class = g_type_class_peek_parent (gobject_class);
 
-	g_object_class_install_property (gobject_class,
-					 PROP_NAME,
-					 g_param_spec_string ("name", "Name",
-							      "The Output's Name",
-							      NULL,
-							      GSF_PARAM_STATIC |
-							      G_PARAM_READABLE));
-	g_object_class_install_property (gobject_class,
-					 PROP_SIZE,
-					 g_param_spec_int64 ("size", "Size",
-							     "The Output's Size",
-							     0, G_MAXINT64, 0,
-							     GSF_PARAM_STATIC |
-							     G_PARAM_READABLE));
-	g_object_class_install_property (gobject_class,
-					 PROP_POS,
-					 g_param_spec_int64 ("position", "Position",
-							     "The Output's Current Position",
-							     0, G_MAXINT64, 0,
-							     GSF_PARAM_STATIC |
-							     G_PARAM_READABLE));
-	g_object_class_install_property (gobject_class,
-					 PROP_CLOSED,
-					 g_param_spec_boolean ("is-closed", "Is Closed",
-							       "Whether the Output is Closed",
-							       FALSE,
-							       GSF_PARAM_STATIC |
-							       G_PARAM_READABLE));
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_NAME,
+		 g_param_spec_string ("name",
+				      _("Name"),
+				      _("The output's name"),
+				      NULL,
+				      GSF_PARAM_STATIC |
+				      G_PARAM_READWRITE));
+
+	/**
+	 * GsfOutput:size:
+	 *
+	 * The current file size.
+	 */
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_SIZE,
+		 g_param_spec_int64 ("size",
+				     _("Size"),
+				     _("The output's size"),
+				     0, G_MAXINT64, 0,
+				     GSF_PARAM_STATIC |
+				     G_PARAM_READABLE));
+
+	/**
+	 * GsfOutput:is-closed:
+	 *
+	 * %TRUE if the output has been closed.
+	 */
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_CLOSED,
+		 g_param_spec_boolean ("is-closed",
+				       _("Is Closed"),
+				       _("Whether the output is closed"),
+				       FALSE,
+				       GSF_PARAM_STATIC |
+				       G_PARAM_READABLE));
+
+	/**
+	 * GsfOutput:position:
+	 *
+	 * The current position in the output.
+	 */
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_POS,
+		 g_param_spec_int64 ("position",
+				     _("Position"),
+				     _("The output's current position"),
+				     0, G_MAXINT64, 0,
+				     GSF_PARAM_STATIC |
+				     G_PARAM_READABLE));
+
+	/**
+	 * GsfOutput:modtime:
+	 *
+	 * The time the output was last updated.  This must be set on object
+	 * construction and represents the timestamp to put on the resulting
+	 * file or #GsfOutfile member.  Not all derived classes will actually
+	 * do anything with this property.
+	 */
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_MODTIME,
+		 g_param_spec_boxed
+		 ("modtime",
+		  _("Modification time"),
+		  _("An optional GDateTime representing the time the output was last changed"),
+		  G_TYPE_DATE_TIME,
+		  GSF_PARAM_STATIC |
+		  G_PARAM_CONSTRUCT_ONLY |
+		  G_PARAM_READWRITE));
+
+	/**
+	 * GsfOutput:container:
+	 *
+	 * The container, optionally %NULL, in which this output lives.
+	 */
+	g_object_class_install_property
+		(gobject_class, PROP_CONTAINER,
+		 g_param_spec_object ("container",
+				      _("Container"),
+				      _("The parent GsfOutfile"),
+				      GSF_OUTFILE_TYPE,
+				      GSF_PARAM_STATIC |
+				      G_PARAM_READWRITE));
 }
 
 GSF_CLASS_ABSTRACT (GsfOutput, gsf_output,
@@ -168,8 +261,7 @@ GSF_CLASS_ABSTRACT (GsfOutput, gsf_output,
  *
  * Give the name of @output.
  *
- * Returns: (transfer none): @output's name in utf8 form, DO NOT FREE
- * THIS STRING
+ * Returns: (transfer none): @output's name in utf8 form.
  **/
 char const *
 gsf_output_name (GsfOutput const *output)
@@ -182,8 +274,7 @@ gsf_output_name (GsfOutput const *output)
  * gsf_output_container:
  * @output:
  *
- * Returns: (transfer none): but does not add a reference to @output's container.
- * 	Potentially %NULL
+ * Returns: (transfer none): @output's container, potentially %NULL.
  **/
 GsfOutfile *
 gsf_output_container (GsfOutput const *output)
@@ -360,7 +451,7 @@ gsf_output_write (GsfOutput *output,
  * gsf_output_error:
  * @output:
  *
- * Returns: the last error logged on the output, or %NULL.
+ * Returns: (transfer none): the last error logged on the output, or %NULL.
  **/
 GError const *
 gsf_output_error (GsfOutput const *output)
@@ -530,6 +621,45 @@ gsf_output_unwrap (GObject *wrapper, GsfOutput *wrapee)
 	return TRUE;
 }
 
+/**
+ * gsf_output_get_modtime:
+ * @output: the output stream
+ *
+ * Returns: (transfer none): A #GDateTime representing when the output
+ * was last modified, or %NULL if not known.
+ */
+GDateTime *
+gsf_output_get_modtime (GsfOutput *output)
+{
+	g_return_val_if_fail (GSF_IS_OUTPUT (output), NULL);
+
+	return g_object_get_data (G_OBJECT (output), MODTIME_ATTR);
+}
+
+/**
+ * gsf_output_set_modtime:
+ * @output: the output stream
+ * @modtime: (transfer none) (allow-none): the new modification time.
+ *
+ * Returns: %TRUE if the assignment was ok.
+ */
+gboolean
+gsf_output_set_modtime (GsfOutput *output, GDateTime *modtime)
+{
+	g_return_val_if_fail (GSF_IS_OUTPUT (output), FALSE);
+
+	if (modtime)
+		modtime = g_date_time_add (modtime, 0); /* Copy */
+
+	/* This actually also works for null modtime.  */
+	g_object_set_data_full (G_OBJECT (output),
+				MODTIME_ATTR, modtime,
+				(GDestroyNotify)g_date_time_unref);
+
+	return TRUE;
+}
+
+
 GQuark
 gsf_output_error_id (void)
 {
@@ -628,7 +758,7 @@ gsf_output_real_vprintf (GsfOutput *output, char const *fmt, va_list args)
 /**
  * gsf_output_puts:
  * @output: A #GsfOutput
- * @line: %null terminated string to write
+ * @line: Nul terminated string to write
  *
  * Like fputs, this assumes that the line already ends with a newline
  *
